@@ -1,17 +1,24 @@
-use super::contracts::{self, engagement_contract, voting, VotingContract};
+use super::contracts::{
+    self, engagement_contract,
+    voting::{self, Proposal},
+    VotingContract,
+};
 use anyhow::Result as AnyResult;
 use cosmwasm_std::{Addr, StdResult};
-use cw3::{
-    Vote, VoteInfo, VoteListResponse, VoteResponse, VoterDetail, VoterListResponse, VoterResponse,
-};
 use cw_multi_test::{AppResponse, Executor};
 use derivative::Derivative;
+use tg3::{
+    Vote, VoteInfo, VoteListResponse, VoteResponse, VoterDetail, VoterListResponse, VoterResponse,
+};
 
 use tg4::Member;
 use tg_bindings_test::TgradeApp;
 
 use crate::{
-    state::{ProposalListResponse, ProposalResponse, RulesBuilder, VotingRules},
+    state::{
+        ProposalInfo, ProposalListResponse, ProposalResponse, RulesBuilder,
+        TextProposalListResponse, VotingRules,
+    },
     ContractError,
 };
 
@@ -32,10 +39,10 @@ impl SuiteBuilder {
         }
     }
 
-    pub fn with_member(mut self, addr: &str, weight: u64) -> Self {
+    pub fn with_member(mut self, addr: &str, points: u64) -> Self {
         self.members.push(Member {
             addr: addr.to_owned(),
-            weight,
+            points,
         });
         self
     }
@@ -117,9 +124,9 @@ impl Suite {
     ) -> AnyResult<AppResponse> {
         let add = add
             .iter()
-            .map(|(addr, weight)| Member {
+            .map(|(addr, points)| Member {
                 addr: (*addr).to_owned(),
-                weight: *weight,
+                points: *points,
             })
             .collect();
 
@@ -133,12 +140,11 @@ impl Suite {
         )
     }
 
-    pub fn propose_detailed(
+    pub fn propose(
         &mut self,
         executor: &str,
         title: &str,
         description: &str,
-        proposal: &str,
     ) -> AnyResult<AppResponse> {
         self.app.execute_contract(
             Addr::unchecked(executor),
@@ -146,14 +152,21 @@ impl Suite {
             &voting::ExecuteMsg::Propose {
                 title: title.to_owned(),
                 description: description.to_owned(),
-                proposal: proposal.to_string(),
+                proposal: Proposal::Text {},
             },
             &[],
         )
     }
 
-    pub fn propose(&mut self, executor: &str, title: &str) -> AnyResult<AppResponse> {
-        self.propose_detailed(executor, title, title, title)
+    pub fn propose_and_execute(
+        &mut self,
+        executor: &str,
+        title: &str,
+        description: &str,
+    ) -> AnyResult<AppResponse> {
+        let prop = self.propose(executor, title, description)?;
+        let id = get_proposal_id(&prop)?;
+        self.execute_proposal(executor, id)
     }
 
     pub fn vote(&mut self, executor: &str, proposal_id: u64, vote: Vote) -> AnyResult<AppResponse> {
@@ -183,8 +196,8 @@ impl Suite {
         )
     }
 
-    pub fn query_proposal(&self, proposal_id: u64) -> StdResult<ProposalResponse<String>> {
-        let prop: ProposalResponse<String> = self.app.wrap().query_wasm_smart(
+    pub fn query_proposal(&self, proposal_id: u64) -> StdResult<ProposalResponse<Proposal>> {
+        let prop: ProposalResponse<Proposal> = self.app.wrap().query_wasm_smart(
             self.voting.clone(),
             &voting::QueryMsg::Proposal { proposal_id },
         )?;
@@ -233,8 +246,8 @@ impl Suite {
         &self,
         start_after: impl Into<Option<u64>>,
         limit: impl Into<Option<usize>>,
-    ) -> StdResult<Vec<ProposalResponse<String>>> {
-        let proposals: ProposalListResponse<String> = self.app.wrap().query_wasm_smart(
+    ) -> StdResult<Vec<ProposalResponse<Proposal>>> {
+        let proposals: ProposalListResponse<Proposal> = self.app.wrap().query_wasm_smart(
             self.voting.clone(),
             &voting::QueryMsg::ListProposals {
                 start_after: start_after.into(),
@@ -244,12 +257,27 @@ impl Suite {
         Ok(proposals.proposals)
     }
 
+    pub fn list_text_proposals(
+        &self,
+        start_after: impl Into<Option<u64>>,
+        limit: impl Into<Option<usize>>,
+    ) -> StdResult<Vec<ProposalInfo>> {
+        let proposals: TextProposalListResponse = self.app.wrap().query_wasm_smart(
+            self.voting.clone(),
+            &voting::QueryMsg::ListTextProposals {
+                start_after: start_after.into(),
+                limit: limit.into().unwrap_or(10),
+            },
+        )?;
+        Ok(proposals.proposals)
+    }
+
     pub fn reverse_proposals(
         &self,
         start_before: impl Into<Option<u64>>,
         limit: impl Into<Option<usize>>,
-    ) -> StdResult<Vec<ProposalResponse<String>>> {
-        let proposals: ProposalListResponse<String> = self.app.wrap().query_wasm_smart(
+    ) -> StdResult<Vec<ProposalResponse<Proposal>>> {
+        let proposals: ProposalListResponse<Proposal> = self.app.wrap().query_wasm_smart(
             self.voting.clone(),
             &voting::QueryMsg::ReverseProposals {
                 start_before: start_before.into(),
@@ -276,6 +304,23 @@ impl Suite {
         Ok(votes.votes)
     }
 
+    pub fn list_votes_by_voter(
+        &self,
+        voter: &str,
+        start_after: impl Into<Option<u64>>,
+        limit: impl Into<Option<usize>>,
+    ) -> StdResult<Vec<VoteInfo>> {
+        let votes: VoteListResponse = self.app.wrap().query_wasm_smart(
+            self.voting.clone(),
+            &voting::QueryMsg::ListVotesByVoter {
+                voter: voter.to_owned(),
+                start_after: start_after.into(),
+                limit: limit.into().unwrap_or(10),
+            },
+        )?;
+        Ok(votes.votes)
+    }
+
     pub fn query_voter(&self, addr: &str) -> Result<VoterResponse, ContractError> {
         let voter: VoterResponse = self.app.wrap().query_wasm_smart(
             self.voting.clone(),
@@ -297,9 +342,9 @@ impl Suite {
     pub fn assert_voters(&mut self, expected: &[(&str, u64)]) {
         let expected: Vec<_> = expected
             .iter()
-            .map(|(addr, weight)| VoterDetail {
+            .map(|(addr, points)| VoterDetail {
                 addr: addr.to_string(),
-                weight: *weight,
+                points: *points,
             })
             .collect();
 
