@@ -2,8 +2,8 @@ use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 
 use cosmwasm_std::{Addr, BlockInfo, Decimal, StdResult, Storage, Uint128};
-use cw3::{Status, Vote};
 use cw_storage_plus::{Item, Map};
+use tg3::{Status, Vote};
 use tg4::Tg4Contract;
 use tg_utils::Expiration;
 
@@ -17,7 +17,7 @@ const PRECISION_FACTOR: u128 = 1_000_000_000;
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 pub struct Config {
     pub rules: VotingRules,
-    // Total weight and voters are queried from this contract
+    // Total points and voters are queried from this contract
     pub group_contract: Tg4Contract,
 }
 
@@ -26,17 +26,79 @@ pub struct Proposal<P> {
     pub title: String,
     pub description: String,
     pub start_height: u64,
+    pub created_by: String,
     pub expires: Expiration,
     pub proposal: P,
     pub status: Status,
     /// pass requirements
     pub rules: VotingRules,
-    // the total weight when the proposal started (used to calculate percentages)
-    pub total_weight: u64,
+    // the total number of points when the proposal started (used to calculate percentages)
+    pub total_points: u64,
     // summary of existing votes
     pub votes: Votes,
 }
 
+impl<P> From<Proposal<P>> for ProposalInfo {
+    fn from(p: Proposal<P>) -> Self {
+        Self {
+            title: p.title,
+            description: p.description,
+        }
+    }
+}
+
+impl<P> Proposal<P> {
+    /// current_status is non-mutable and returns what the status should be.
+    /// (designed for queries)
+    pub fn current_status(&self, block: &BlockInfo) -> Status {
+        let mut status = self.status;
+
+        // if open, check if voting is passed or timed out
+        if status == Status::Open && self.is_passed(block) {
+            status = Status::Passed;
+        }
+        if status == Status::Open && self.expires.is_expired(block) {
+            status = Status::Rejected;
+        }
+
+        status
+    }
+
+    /// update_status sets the status of the proposal to current_status.
+    /// (designed for handler logic)
+    pub fn update_status(&mut self, block: &BlockInfo) {
+        self.status = self.current_status(block);
+    }
+
+    // returns true iff this proposal is sure to pass (even before expiration if no future
+    // sequence of possible votes can cause it to fail)
+    pub fn is_passed(&self, block: &BlockInfo) -> bool {
+        let VotingRules {
+            quorum,
+            threshold,
+            allow_end_early,
+            ..
+        } = self.rules;
+
+        // we always require the quorum
+        if self.votes.total() < votes_needed(self.total_points, quorum) {
+            return false;
+        }
+        if self.expires.is_expired(block) {
+            // If expired, we compare Yes votes against the total number of votes (minus abstain).
+            let opinions = self.votes.total() - self.votes.abstain;
+            self.votes.yes >= votes_needed(opinions, threshold)
+        } else if allow_end_early {
+            // If not expired, we must assume all non-votes will be cast as No.
+            // We compare threshold against the total points (minus abstain).
+            let possible_opinions = self.total_points - self.votes.abstain;
+            self.votes.yes >= votes_needed(possible_opinions, threshold)
+        } else {
+            false
+        }
+    }
+}
+
 /// Note, if you are storing custom messages in the proposal,
 /// the querier needs to know what possible custom message types
 /// those are in order to parse the response
@@ -45,11 +107,12 @@ pub struct ProposalResponse<P> {
     pub id: u64,
     pub title: String,
     pub description: String,
+    pub created_by: String,
     pub proposal: P,
     pub status: Status,
     pub expires: Expiration,
     pub rules: VotingRules,
-    pub total_weight: u64,
+    pub total_points: u64,
     pub votes: Votes,
 }
 
@@ -58,6 +121,11 @@ pub struct ProposalListResponse<P> {
     pub proposals: Vec<ProposalResponse<P>>,
 }
 
+#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+pub struct TextProposalListResponse {
+    pub proposals: Vec<ProposalInfo>,
+}
+
 #[derive(Serialize, Deserialize, Clone, PartialEq, Debug, JsonSchema)]
 pub struct VotingRules {
     /// Length of voting period in days.
@@ -105,7 +173,7 @@ impl RulesBuilder {
     pub fn new() -> Self {
         Self {
             voting_period: 14,
-            quorum: Decimal::percent(1),
+            quorum: Decimal::percent(20),
             threshold: Decimal::percent(50),
             allow_end_early: true,
         }
@@ -142,7 +210,7 @@ impl Default for RulesBuilder {
     }
 }
 
-// weight of votes for each option
+// points of votes for each option
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 pub struct Votes {
     pub yes: u64,
@@ -158,90 +226,38 @@ impl Votes {
     }
 
     /// create it with a yes vote for this much
-    pub fn yes(init_weight: u64) -> Self {
+    pub fn yes(init_points: u64) -> Self {
         Votes {
-            yes: init_weight,
+            yes: init_points,
             no: 0,
             abstain: 0,
             veto: 0,
         }
     }
 
-    pub fn add_vote(&mut self, vote: Vote, weight: u64) {
+    pub fn add_vote(&mut self, vote: Vote, points: u64) {
         match vote {
-            Vote::Yes => self.yes += weight,
-            Vote::Abstain => self.abstain += weight,
-            Vote::No => self.no += weight,
-            Vote::Veto => self.veto += weight,
-        }
-    }
-}
-
-impl<P> Proposal<P> {
-    /// current_status is non-mutable and returns what the status should be.
-    /// (designed for queries)
-    pub fn current_status(&self, block: &BlockInfo) -> Status {
-        let mut status = self.status;
-
-        // if open, check if voting is passed or timed out
-        if status == Status::Open && self.is_passed(block) {
-            status = Status::Passed;
-        }
-        if status == Status::Open && self.expires.is_expired(block) {
-            status = Status::Rejected;
-        }
-
-        status
-    }
-
-    /// update_status sets the status of the proposal to current_status.
-    /// (designed for handler logic)
-    pub fn update_status(&mut self, block: &BlockInfo) {
-        self.status = self.current_status(block);
-    }
-
-    // returns true iff this proposal is sure to pass (even before expiration if no future
-    // sequence of possible votes can cause it to fail)
-    pub fn is_passed(&self, block: &BlockInfo) -> bool {
-        let VotingRules {
-            quorum,
-            threshold,
-            allow_end_early,
-            ..
-        } = self.rules;
-
-        // we always require the quorum
-        if self.votes.total() < votes_needed(self.total_weight, quorum) {
-            return false;
-        }
-        if self.expires.is_expired(block) {
-            // If expired, we compare Yes votes against the total number of votes (minus abstain).
-            let opinions = self.votes.total() - self.votes.abstain;
-            self.votes.yes >= votes_needed(opinions, threshold)
-        } else if allow_end_early {
-            // If not expired, we must assume all non-votes will be cast as No.
-            // We compare threshold against the total weight (minus abstain).
-            let possible_opinions = self.total_weight - self.votes.abstain;
-            self.votes.yes >= votes_needed(possible_opinions, threshold)
-        } else {
-            false
+            Vote::Yes => self.yes += points,
+            Vote::Abstain => self.abstain += points,
+            Vote::No => self.no += points,
+            Vote::Veto => self.veto += points,
         }
     }
 }
 
 // this is a helper function so Decimal works with u64 rather than Uint128
 // also, we must *round up* here, as we need 8, not 7 votes to reach 50% of 15 total
-fn votes_needed(weight: u64, percentage: Decimal) -> u64 {
-    let applied = percentage * Uint128::new(PRECISION_FACTOR * weight as u128);
+fn votes_needed(points: u64, percentage: Decimal) -> u64 {
+    let applied = percentage * Uint128::new(PRECISION_FACTOR * points as u128);
     // Divide by PRECISION_FACTOR, rounding up to the nearest integer
     ((applied.u128() + PRECISION_FACTOR - 1) / PRECISION_FACTOR) as u64
 }
 
-// we cast a ballot with our chosen vote and a given weight
+// we cast a ballot with our chosen vote and a given points
 // stored under the key that voted
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 pub struct Ballot {
-    pub weight: u64,
+    pub points: u64,
     pub vote: Vote,
 }
 
@@ -251,10 +267,19 @@ pub const PROPOSAL_COUNT: Item<u64> = Item::new("proposal_count");
 
 // multiple-item map
 pub const BALLOTS: Map<(u64, &Addr), Ballot> = Map::new("votes");
+pub const BALLOTS_BY_VOTER: Map<(&Addr, u64), Ballot> = Map::new("votes_by_voter");
 pub fn proposals<'m, P>() -> Map<'m, u64, Proposal<P>> {
     Map::new("proposals")
 }
 
+#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+pub struct ProposalInfo {
+    pub title: String,
+    pub description: String,
+}
+
+pub const TEXT_PROPOSALS: Map<u64, ProposalInfo> = Map::new("text_proposals");
+
 pub fn next_id(store: &mut dyn Storage) -> StdResult<u64> {
     let id: u64 = PROPOSAL_COUNT.may_load(store)?.unwrap_or_default() + 1;
     PROPOSAL_COUNT.save(store, &id)?;
